/* 
 * CSE142, 99 Winter
 * Homework 5
 * Valentine's Day
 *
 * Author:     Luke Meyers
 * Student ID: 9828787
 * Section:    AI
 *	
 * STARTER PROGRAM
 *
 * Greeting Card Designer (Ultimate version)
 *
 *  The GP142 graphics window shows some buttons and the inside
 *  or outside (cover) of a greeting card.  The idea is that you 
 *  can design what you want to card to look like by adding and moving
 *  objects: 
 *    messages, message color and position
 *    shapes,   shape color and position
 *
 *   To place a message or a greeting, you click on an appropriate button, and 
 *   then you can click on the desired position in the card.  
 */

#include "gp142.h"
#include <math.h>
#include <time.h>

#define TRUE    		1
#define FALSE   		0

/*
 * The state, or status you are in determines what happens when you click 
 * the card:
 * ADD SHAPE   -- you click the card, and a new object is placed there.
 *                You get into this mode by clicking an ADD button.
 *                After doing an ADD, you are put into SELECTED mode.
 * SELECTED    -- An object is selected. In this mode, the type 
 *                rotation applies to thecurrent object.
 * CHOOSE      -- The user is being prompted to click on an object to
 *                select it.
 */
#define STATUS_ADD_SHAPE 1
#define STATUS_SELECTED 2
#define STATUS_CHOOSE 3
#define STATUS_ADD_MESSAGE 4
#define STATUS_WAIT 5

#define BUTTON_HEIGHT  	35
#define BUTTON_WIDTH 	115

#define NUM_BUTTONS 	9
#define SELECTED_COLOR  YELLOW

/* Position of the card in the window */
#define PAGE_LLX (-GP142_XMAX + BUTTON_WIDTH +60)
#define PAGE_LLY (-GP142_YMAX + 100)
#define PAGE_WIDTH (GP142_XMAX - PAGE_LLX - 80)
#define PAGE_HEIGHT (GP142_YMAX - PAGE_LLY - 50)

/* For many of the GP142 functions, a line width of
 * zero means to fill the shape being drawn.
 */
#define FILL 0
#define NOFILL 1

/* Types of objects we support. */
#define OBJECT_NONE 0
#define OBJECT_SHAPE 1
#define OBJECT_MESSAGE 2

#define MAX_SHAPES 4
#define FIRST_SHAPE_NUM 0
#define MAX_MESSAGES 8

/* These are the kinds of shapes we support. */
#define SHAPE_HEART 0
#define SHAPE_STAR  1
#define SHAPE_CONFETTI 2
#define SHAPE_ARROW 3

#define HEART_WIDTH 35

/*Possible places for mouse clicks*/
#define CLICK_BUTTON 1  /* You clicked on SOME button (doesn't say which one)*/
#define CLICK_CARD 2    /* You clicked in the card area */
#define CLICK_OUTSIDE 3	/* You clicked outside the card area, not on a button*/	

#define MAX_OBJECTS 256 /* Maximum number of objects user can add */


/*******************  FUNCTION PROTOTYPES **************************/

/* Drawing functions */
void DrawBlankScreen (void);
void DrawButtons(int last_button);
void DrawCard (int background_color, int status, int object_count, 
			   int object_type[], int object_num[], 
			   int object_color[], int object_x[], int object_y[],
            int selected, int frame_count);
void DrawShape(int color, int shape_type, int bottom_x, int bottom_y);
void DrawHeart (int x, int y, int color, int width);
void DrawStar (int x, int y, int color);
void DrawConfetti (int x, int y);
void DrawArrow (int x, int y, int color);
void DrawMessage(int object_num, int color, int x, int y, int font_size);

/* Mouse click handling functions */
void HandleMouseClick(int mouse_x, int mouse_y, int *background_color, 
					  int *status, int *object_count, int object_type[], 
					  int object_num[], int object_color[], 
					  int object_x[], int object_y[], int *last_button,
                 int *selected);
int ClassifyMouseClick(int mouse_x, int mouse_y);
void HandleButton(int x, int y, int *background_color, int *status,
				  int object_count, int *object_type, int *object_num,
              int *last_button, int *selected, int object_color[]);

/* Add a shape to the card. */
void AddShape(int x, int y, int *status, int *object_count, 
			   int *object_type, int *object_num, int *object_color, 
			   int *object_x, int *object_y, int *selected, int *last_button);

/*****************************************************************************

     MAIN

*****************************************************************************/

int main(void)
{
	int quit = FALSE;              /* becomes TRUE to exit program */
	int mouse_x, mouse_y;          /* coordinates of a mouse click */
	char key_hit;                  /* which key was hit            */
   int selected = 0;              /* index of currently selected object */	
	
	/*variables and their default values*/
	int background_color = PURPLE;
	int status = STATUS_WAIT;
	
	/* Each object will have a number telling what kind of object it is, a color,
	 * and an x and y coordinate.  You'll need to make these into arrays so you
	 * can support more than one object.
	 */
	int object_count = 0;           /* How many objects on the page? */
	int object_type [MAX_OBJECTS];  /* Can also be a shape           */
	int object_num [MAX_OBJECTS];   /* If type is shape, either 
									            heart or star                 */
	int object_color [MAX_OBJECTS]; /* Color of object               */
	int object_x [MAX_OBJECTS];     /* Position of object            */
	int object_y [MAX_OBJECTS];
	
	int last_button = -1;           /* Keep track of the last button clicked.
	                                   This would be usefull when highlighting
	                                   the last button the user clicked */

	int event; /*the most recent GP142 event*/
   int frame_count = 0; /* number of completed animation frames */
	
	time_t t; /*needed only when initializing a random number generator*/
	

	/*
	 * INITIALIZATIONS
	 */
	srand (time(&t));	    /* Seed the random number generator. This is 
							   something you should do if you plan to use 
							   random numbers. Leave it here in your program 
							   and forget about it.*/
	

	/* These GP142 initializations don't need to be changed */
	GP142_open();           /* Open and init the GP142 Graphics Window      */
	GP142_show_text(0);     /* Hide the text window                         */
	GP142_logging(LOG_OFF); /* Change to LOG_ON if you want it.             */
	
	GP142_clear();	        /* This will clear the window & paint it white. */
	GP142_animate(ANI_RUN); /* Enable periodic events                       */
	
	
	
	/*
	 * The main event loop:
	 * --- ---- ----- -----
	 * wait for the next user action, such as a mouse click or keyboard
	 * keypress, or a periodic update event (allowing us to display the
	 * next frame of an animation -- not used directly in this program)
	 * and call the appropriate function to handle it.  Repeat this until 
	 * the user signals "quit".
	 */
	while (!quit) {  /*this is the famous "event loop" */
		
		/*start by redrawing everything */
      frame_count++;
		DrawBlankScreen ();
		DrawCard (background_color, status, object_count, 
			      object_type, object_num, object_color, object_x, object_y,
               selected, frame_count);
		DrawButtons(last_button);	  /* Show the buttons...*/ 
		
		/*Wait for something to happen */
		event = GP142_await_event(&mouse_x, &mouse_y, &key_hit);
		
		/* Something just happened.  Figure out what it was, and react 
		   appropriately. */
		switch (event) {
		case GP142_QUIT:
			quit = TRUE;           /* set flag to terminate loop           */
			break;
			
		case GP142_KBD:  /*A key was pressed */
			/*not needed for this program. Use this if user will need to 
			  hit a key. Note that scanf CANNOT be used with GP142.  If you 
			  use this, the key character value is in variable "key_hit"*/
			break;
			
		case GP142_MOUSE:
			/* A mouse click has occurred. Call the mouse handler to 
			   decide where the click was and react as needed. */
			HandleMouseClick(mouse_x, mouse_y, &background_color, &status, 
					         &object_count, object_type, 
							 object_num, object_color, 
							 object_x, object_y, &last_button, &selected);			
			break;
			
		case GP142_PERIODIC:
			/*An event has occurred simply because some time has passed,*/
			/*not currently used in the program*/		
			break;

		default:
			
			break;
			
		}  /*end switch */
		
	}  /* end event loop */
	
	
	/*
	 * WRAP-UP
	 */
	GP142_close();         /* clean up and close graphics window   */
	
	return 0;
}  /*end main */

/*****************************************************************************

     FUNCTIONS

*****************************************************************************/
/* 
 * For a blank screen just draw a white rectangle that covers the whole
 * GP142 window.
 */
void DrawBlankScreen (void) {
	int font_size = 14;

	GP142_rectangleXY (CHALK,
					-GP142_XMAX, -GP142_YMAX,
					GP142_XMAX, GP142_YMAX,
					FILL);

} /*end DrawBlankScreen */


/* 
 * Draws the buttons 
 */
void DrawButtons(int last_button)
{
	char b_labels [NUM_BUTTONS][25] = 
   {
		"Background color",
		"Next, please",
      "Add a shape",
      "Select an object",
      "Add a message",
      "Object color",
      "Cycle colors",
      "Deselect object",
      "Delete object"
   };
	
	int i, text_offset = -GP142_XMAX + 9, font_size = 12;
	int y, label_y;
	
	/* draw rectangles and labels for each button */
	for(i = 0; i < NUM_BUTTONS; i++) {
		y = GP142_YMAX - i * BUTTON_HEIGHT;
		
		GP142_rectangleXY(BLACK,
			-GP142_XMAX + 6,
			y - 7,
			-GP142_XMAX + BUTTON_WIDTH,
			y - BUTTON_HEIGHT - 2, 2);

      GP142_rectangleXY(WHITE,
			-GP142_XMAX + 6,
			y - 6,
			-GP142_XMAX + BUTTON_WIDTH -2,
			y - BUTTON_HEIGHT , 2);

      GP142_rectangleXY(LT_GRAY,
			-GP142_XMAX + 7,
			y - 8,
			-GP142_XMAX + BUTTON_WIDTH - 4,
			y - BUTTON_HEIGHT+2, FILL);

		
		
		label_y = y - (int)((double)BUTTON_HEIGHT/1.5);
		
		if (i == last_button)
      {
         GP142_rectangleXY(MED_GRAY,
			-GP142_XMAX + 7,
			y - 8,
			-GP142_XMAX + BUTTON_WIDTH - 4,
			y - BUTTON_HEIGHT+2, FILL);

			GP142_printfXY(SELECTED_COLOR, text_offset, label_y, font_size + 2,
         b_labels[i]);	
      }
		else
			GP142_printfXY(BLACK, text_offset, label_y, font_size, b_labels[i]);	
	}	
}


/*
 * Render (draw) the page and all its contents.
 */
void DrawCard (int background_color, int status, int object_count, 
			   int object_type[], int object_num[], 
			   int object_color[], int object_x[], int object_y[],
            int selected, int frame_count)
{
	int font_size = 18;
   int i;
   int used_color;
	
	/*
	 * Draw the background of the card.
	 */
/* Old rectangular background commented out 
 *    GP142_rectangleXY (background_color + 1,
 *	   PAGE_LLX, PAGE_LLY,
 * 	PAGE_LLX+ PAGE_WIDTH, PAGE_LLY+ PAGE_HEIGHT,
 *		FILL);
 */

/*
 * Draw heart-shaped background (may look wrong on other systems, needs testing
 */
   GP142_circleXY (background_color,
      PAGE_LLX + (PAGE_WIDTH * 1/4) + 5,
      PAGE_LLY + (PAGE_HEIGHT / 2) + 50,
      PAGE_WIDTH / 4);
	GP142_circleXY (background_color, 
      PAGE_LLX + (PAGE_WIDTH * 3/4) - 5,
      PAGE_LLY + (PAGE_HEIGHT / 2) + 50,
      PAGE_WIDTH / 4);
   GP142_rectangleXY (background_color,
      PAGE_LLX + (PAGE_WIDTH / 2) - 20,
      PAGE_LLY + (PAGE_HEIGHT / 2) + 5,
      PAGE_LLX + (PAGE_WIDTH / 2) + 20,
      PAGE_LLY + (PAGE_HEIGHT / 2) + 40,
      FILL);
   GP142_triangleXY (background_color,
      PAGE_LLX + (PAGE_WIDTH / 2),
      PAGE_LLY + 10,
      PAGE_LLX + PAGE_WIDTH - 14,
      PAGE_LLY + (PAGE_HEIGHT / 2) + 10,
      PAGE_LLX + 14,
      PAGE_LLY + (PAGE_HEIGHT / 2) + 10,
      FILL);



	/* 
	 * Draw all defined objects on the card
	 */
   for (i = 0; i < object_count; i++)
   {
      if ((i == selected) && (frame_count % 5 == 0))
         used_color = background_color;
      else
      {
         if (object_color[i] < 0) 
            used_color = rand() % MAX_COLORS;
         else used_color = object_color[i] % MAX_COLORS;
      }

      if (object_type[i] == OBJECT_SHAPE) 
      {
	   	DrawShape (object_num[i], used_color, 
            object_x[i], object_y[i]);
   	}
	   if (object_type[i] == OBJECT_MESSAGE)
      {
         DrawMessage(object_num[i], used_color, 
            object_x[i],object_y[i], font_size);
      }
   }
	/*
	 * Write an appropriate status message below the background
	 */
	switch (status) {
	case STATUS_ADD_SHAPE:
		GP142_printfXY (BLACK, PAGE_LLX, PAGE_LLY-40, font_size,
			"Click on the card to add a shape.");
		break;
	case STATUS_SELECTED:
		GP142_printfXY (RED, PAGE_LLX, PAGE_LLY-40, font_size,
			"Object selected. Click button to change shape or type.");
		break;
   case STATUS_CHOOSE:
      GP142_printfXY (BLUE, PAGE_LLX, PAGE_LLY-40, font_size,
         "Click near the center of an object to select it.");
      break;
   case STATUS_ADD_MESSAGE:
      GP142_printfXY (BLACK, PAGE_LLX, PAGE_LLY-40, font_size,
         "Click on the card to add a message.");
      break;
   case STATUS_WAIT:
      GP142_printfXY (BLACK, PAGE_LLX, PAGE_LLY-40, font_size,
         "Greeting Card Designer V1.0 by Luke Meyers");
      break;
	default:
		GP142_printfXY (BLACK, PAGE_LLX, PAGE_LLY-40, font_size,
			"Status error -- notify programmer %i", status);
		break;
	} /*end switch */
	
	
} /*end DrawCard */



/*
 * Mouse clicks occur in one of three regions of interest: on buttons, 
 * on the card, or anywhere else.
 */
int  ClassifyMouseClick(int mouse_x, int mouse_y)
{
  if ((mouse_x < -GP142_XMAX + BUTTON_WIDTH) &&
      (mouse_y < GP142_YMAX) &&
      (mouse_y > (GP142_YMAX - NUM_BUTTONS*BUTTON_HEIGHT)))
    return CLICK_BUTTON;

  else 
  if (mouse_x >= PAGE_LLX && mouse_x < PAGE_LLX + PAGE_WIDTH
      && mouse_y >= PAGE_LLY && mouse_y < PAGE_LLY + PAGE_HEIGHT)
  {
   /* 
    * This lengthy bit of code tests whether the mouse click was within the
    * permiter of the heart-shaped background.
    */
     if (
            (
            pow ( (mouse_x - (PAGE_LLX + (PAGE_WIDTH * 1 / 4) + 5)), 2 ) +
            pow ((mouse_y - (PAGE_LLY + (PAGE_HEIGHT / 2) + 50)), 2) <
            pow (PAGE_WIDTH / 4, 2)
            )
        ||
           (
            pow ( ( mouse_x - (PAGE_LLX + (PAGE_WIDTH * 3 / 4) - 5) ), 2 ) +
            pow ( ( mouse_y - (PAGE_LLY + (PAGE_HEIGHT  / 2) + 50 ) ), 2) <
            pow (PAGE_WIDTH / 4, 2)
           )
        ||
           (
            mouse_x < (PAGE_LLX + PAGE_WIDTH / 2)  &&
            mouse_x > (PAGE_LLX + 5) &&
            mouse_y < PAGE_LLY + (PAGE_HEIGHT / 2) + 10 &&
            mouse_y > PAGE_LLY + 10 + (PAGE_HEIGHT / 2) +
                        (
                           (-PAGE_HEIGHT/2) / ((PAGE_WIDTH / 2) - 14) * 
                          (mouse_x - (PAGE_LLX + 14))
                        )
           )
        ||
           (
            mouse_x > (PAGE_LLX + PAGE_WIDTH / 2)  &&
            mouse_x < (PAGE_LLX + PAGE_WIDTH - 5) &&
            mouse_y < PAGE_LLY + (PAGE_HEIGHT / 2) + 10 &&
            mouse_y > PAGE_LLY + 10  + (PAGE_HEIGHT / 2) +
                        (
                          (PAGE_HEIGHT/2) / ((PAGE_WIDTH/2) - 14) *
                          (mouse_x - (PAGE_LLX + PAGE_WIDTH - 14))
                        ) 
           )
       )
       return CLICK_CARD;

  }
  return CLICK_OUTSIDE;

}


/*
 * Decide where a mouse click occured, and react appropriately.
 */
void HandleMouseClick(int mouse_x, int mouse_y, int *background_color, 
					  int *status, int *object_count, int object_type[], 
					  int object_num[], int object_color[], 
					  int object_x[], int object_y[], int *last_button, int *selected)
{

	int mouse_click;
   int i;
	mouse_click = ClassifyMouseClick(mouse_x, mouse_y);
			
	if (mouse_click == CLICK_BUTTON) 
   {
		/* A button has been clicked.  Call a function to handle it*/
		/* most of the variables passed as args will NOT change*/
		HandleButton(mouse_x, mouse_y, background_color, status, 
					 *object_count, object_type, object_num, last_button,
                selected, object_color);
	} 
   else if (mouse_click == CLICK_CARD) 
   {
      switch(*status)
      {
      case STATUS_ADD_SHAPE:
         /* Add a shape */
		   AddShape(mouse_x, mouse_y, status, object_count, 
			      object_type, object_num, object_color, 
			      object_x, object_y, selected, last_button);
         break;
      case STATUS_SELECTED:
         object_x[*selected] = mouse_x;
         object_y[*selected] = mouse_y;
         break;
      case STATUS_CHOOSE:
         /* check to see if user clicked close to the center of an object
          * and make the first object close enough to the click become the
          * new selection
          */
         for (i = 0; i < *object_count; i++)
         {
            if ((abs(mouse_x - object_x[i]) < 15) &&
                (abs(mouse_y - object_y[i]) < 15))
            {
               *selected = i;
               *status = STATUS_SELECTED;
               break;
            }
         }
         break;
      case STATUS_ADD_MESSAGE:
         /* Add a message */
         AddShape(mouse_x, mouse_y, status, object_count, 
			object_type, object_num, object_color, 
			object_x, object_y, selected, last_button);
         object_type[*selected] = OBJECT_MESSAGE;
         break;
      }
	}
	/* Ignore any other mouse clicks */
}


/* 
 * Figure out which button was clicked, if any, and do the right 
 * thing.
 */
void HandleButton(int x, int y, int *background_color, int *status,
				  int object_count, int object_type[], int object_num[],
              int *last_button, int *selected, int object_color[])
{
	int buttonNum = 0;
   int i = *selected;
	
	/* If we're outside of this area, we could not have clicked on
	* a button.
	*/
	if (x > -GP142_XMAX + BUTTON_WIDTH)  
		return ;	/*That's an error situation*/
	
	/* We know the mouse clicked in the column of buttons. Calculate
  	 * which one.
    */
	buttonNum = (GP142_YMAX - y)/BUTTON_HEIGHT;
   *last_button = buttonNum;
	
	switch(buttonNum) {
	case 0:
		/* Change the background color */
		*background_color = (*background_color + 1) % MAX_COLORS;
		break;
	case 1:
		/* Change the type of the currently selected object */
		if ((object_count >= 1) && (object_type[*selected] == OBJECT_SHAPE))
      {
			object_num[*selected] = (object_num[*selected] + 1) % MAX_SHAPES;
		}
      else if((object_count >= 1) && (object_type[*selected] == OBJECT_MESSAGE))
      {
			object_num[*selected] = (object_num[*selected] + 1) % MAX_MESSAGES;
      }
		break;
   case 2:
      /* Add a new shape */
      *status = STATUS_ADD_SHAPE;
      *selected = -1;
      break;
   case 3:
      /* prompt user to click on a shape */
      *status = STATUS_CHOOSE;
      break;
   case 4:
      /* add a new message */
      *status = STATUS_ADD_MESSAGE;
      break;
   case 5:
      /* change object color */
      object_color[*selected] += 1;
      *status = STATUS_SELECTED;
      break;
   case 6:
      /* toggle cycling colors for current object */
      if (*status == STATUS_SELECTED)
      {
         if (object_color[*selected] >= 0)
            object_color[*selected] = -1;
         else object_color[*selected] = rand() % MAX_COLORS;
      }
      break;
   case 7:
      /* deselect current object */
      *selected = -1;
      *status = STATUS_WAIT;
      break;
   case 8:
      /* delete current object */
      if (*status == STATUS_SELECTED)
      {
         object_type[*selected] = OBJECT_NONE;
         *selected = -1;
         *status = STATUS_WAIT;
      }
      break;
	default:
		*status = 999;
		/*Bad news if this is ever reached! */
		break;
	}
}


/* If there's room for another shape, add it to the card. */
void AddShape(int x, int y, int *status, int *object_count, 
			   int object_type[], int object_num[], int object_color[], 
			   int object_x[], int object_y[], int *selected, int *last_button)
{
   int i;
	if (*object_count < MAX_OBJECTS) 
   {
      *object_count += 1;
      i = *object_count - 1;
      *selected = i;
      *last_button = -1;
		*status = STATUS_SELECTED;
		object_type[i] = OBJECT_SHAPE;
		object_num[i] = SHAPE_HEART;
		object_color[i] = RED;
		object_x[i] = x;
		object_y[i] = y;
	}

}

/*
 * Draw a heart on the card. A heart will be made by two overlapping
 * circles and a triangle.
 */

void DrawHeart (int x, int y, int color, int width) 
{
	int h = (width/2) + (rand () %4);

	GP142_circleXY (color, x - h + 5, y, h);
	GP142_circleXY (color, x + h - 5, y, h);
	GP142_triangleXY (color, 
                  x - (h * 2) + 5, y - width/5,
						x + (h * 2) - 5, y - width/5,
						x, y - (h * 2) - 5,
                  FILL);
}


/*
 * Draw a star on the card.
 */
#define STARPOINTS 5
#define STARRADIUS 40
#define PI 3.1416
void DrawStar (int x, int y, int color) 
{
	int sp;
	double angle;
	int x1, y1;

	sp = STARPOINTS + (rand () %3);
	for (angle = 0.0; angle < 2 * PI; angle = angle + (2*PI)/sp) {
		x1 = (int) (cos (angle) * STARRADIUS);
		y1 = (int) (sin (angle) * STARRADIUS);
		GP142_lineXY (color, x+x1, y+y1, x-x1, y-y1, 1);

	}
}


/*
 * Draw a patch of confetti
 */
#define CONFETTI_PCS 12
#define CONFETTI_SIZE 35
#define PC_SIZE 3
void DrawConfetti (int x, int y)
{
   int pcs = CONFETTI_PCS + (rand()%4);
   int i;

   int pcx,    /* coordinates of individual pieces */
       pcy;
   for (i = 0; i < pcs; i++)
   {
      pcx = (x - CONFETTI_SIZE) + (rand() % (CONFETTI_SIZE * 2));
      pcy = (y - CONFETTI_SIZE) + (rand() % (CONFETTI_SIZE * 2));
      GP142_circleXY(rand() % MAX_COLORS, pcx, pcy, PC_SIZE);
   }
}
                     
/*
 * Draw an arrow
 */
#define ARROW_WIDTH 50
#define ARROW_HEIGHT 50
void DrawArrow (int x, int y, int color)
{
   int lx = x - (ARROW_WIDTH / 2),
       ux = x + (ARROW_WIDTH / 2),
       ly = y - (ARROW_HEIGHT / 2),
       uy = y + (ARROW_HEIGHT / 2);
   GP142_lineXY(color, lx, ly, ux, uy, 3);
   GP142_triangleXY(color, lx, ly, lx + 15, ly, lx, ly + 15, FILL);
   GP142_lineXY(color, ux, uy, ux + 10, uy, 3);
   GP142_lineXY(color, ux, uy, ux, uy + 10, 3);

   GP142_lineXY(color, ux - 10, uy - 10, ux - 10, uy, 3);
   GP142_lineXY(color, ux - 10, uy - 10, ux, uy - 10, 3);

   GP142_lineXY(color, ux - 5, uy - 5, ux - 5, uy + 5, 3);
   GP142_lineXY(color, ux - 5, uy - 5, ux + 5, uy - 5, 3);

   
}

/*
 * Draw one shape.
 */
void DrawShape(int shape_type, int color, int bottom_x, int bottom_y) 
{

	switch ((shape_type - FIRST_SHAPE_NUM) % MAX_SHAPES) 
   {
	   case SHAPE_HEART:
		   DrawHeart (bottom_x, bottom_y, color, HEART_WIDTH);
		   break;
	   case SHAPE_STAR:
		   DrawStar (bottom_x, bottom_y, color);
		   break;
      case SHAPE_CONFETTI:
         DrawConfetti (bottom_x, bottom_y);
         break;
      case SHAPE_ARROW:
         DrawArrow (bottom_x, bottom_y, color);
         break;
	   default:
		   /* Unknown shape selected */
		   break;
	} /*end switch*/

}


/* draw one message on the card */
void DrawMessage(int object_num, int color, int x, int y, int font_size) 
{
	switch (object_num % MAX_MESSAGES) 
   {
      case 0 :
         GP142_printfXY (color, x, y,
            font_size, "I love you!");
         break;
      case 1 :
         GP142_printfXY (color, x, y,
            font_size, "You're my favorite!");
         break;
      case 2 :
         GP142_printfXY (color, x, y,
            font_size, "Be mine!");
         break;
      case 3 :
         GP142_printfXY (color, x, y,
            font_size, "Happy Valentine's Day!");
         break;
      case 4 :
         GP142_printfXY (color, x, y,
            font_size, "Can't get enough of your love");
         break;
      case 5 :
         GP142_printfXY (color, x, y,
            font_size, "Kiss me");
         break;
      case 6 :
         GP142_printfXY (color, x, y,
            font_size, "You're my one and only");
         break;
      case 7 :
         GP142_printfXY (color, x, y,
            font_size, "XOXOXOX");
         break;

      default:
         GP142_printfXY (color, x, y,
            font_size, "Error!");
	}
}

